/*
 * util.h - Utility functions
 *
 * Written 2002,2003 by Werner Almesberger, Caltech Netlab FAST project
 */

#ifndef UTIL_H
#define UTIL_H

#include <stdlib.h>
#include <stdio.h>
#include <string.h>


/*
 * Temporary local variables in macros are prefixed with _ to avoid -Wshadow
 * complaints.
 */

/* ----- Dynamic memory allocation ----------------------------------------- */


#ifdef FIND_MEMORY_LEAKS

void register_alloc(const void *p,int size,const char *file,int line,
  const char *function);
void register_free(const void *p);
#define free(p) do { register_free(p); free(p); } while (0)
#define realloc(p,size) \
  ({ void *_new; if (p) register_free(p); \
    _new = realloc(p,size); \
    if (_new) register_alloc(_new,size,__FILE__,__LINE__,__FUNCTION__); \
    _new; })
void report_memory_leaks(void);

#else /* FIND_MEMORY_LEAKS */

#define register_alloc(p,size,file,line,function)
#define report_memory_leaks()

#endif /* !FIND_MEMORY_LEAKS */


#define stralloc(s) ({ char *_tmp = strdup(s); \
    if (!_tmp) { perror("strdup"); exit(1); }; \
    register_alloc(_tmp,strlen(_tmp)+1,__FILE__,__LINE__,__FUNCTION__); \
    _tmp; })

#define alloc(size) ({ void *_tmp = malloc(size); \
    if (!_tmp) { perror("malloc"); exit(1); }; \
    register_alloc(_tmp,size,__FILE__,__LINE__,__FUNCTION__); \
    _tmp; })

#define alloc_t(t) ((t *) alloc(sizeof(t)))


/* ----- Sign extension ---------------------------------------------------- */


#define signex_tn(out_t,in,n) \
    ({ out_t ones = 0; ones = ~ones; /* portable ~0 */ \
	out_t _tmp = (in); \
	sizeof(out_t) <= (n) ? _tmp : \
	  _tmp & (1 << ((n)*8-1)) ? _tmp | (ones << (8*(n))) : _tmp; })
#define signex_n(out,in,n) \
    ({ typeof(out) ones = 0; ones = ~ones; /* portable ~0 */ \
	typeof(out) _tmp = (in); \
	out = sizeof(out) <= (n) ? _tmp : \
	  _tmp & (1 << ((n)*8-1)) ? _tmp | (ones << (8*(n))) : _tmp; })
#define signex(out,in) signex_n(out,in,sizeof(in))

#endif /* UTIL_H */
